# Copyright 2025 Terradue
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from . import BaseCWLtypes2OGCConverter
from cwl_utils.parser import load_document_by_uri, Process
from datetime import datetime
from loguru import logger
from pathlib import Path

import click
import json
import time


@click.command(context_settings={"show_default": True})
@click.argument(
    "source",
    type=click.Path(path_type=Path, exists=True, readable=True, resolve_path=True),
    required=True,
)
@click.option("--workflow-id", required=True, help="ID of the workflow")
@click.option(
    "--output",
    type=click.Path(path_type=Path),
    required=False,
    default="process.json",
    help="The output file path",
)
def main(source: Path, workflow_id: str, output: Path):
    start_time = time.time()

    logger.debug(f"Loading {workflow_id} from CWL document on {source}...")

    workflow: Process = load_document_by_uri(f"{source}#{workflow_id}")

    logger.debug(f"CWL document from {source} successfully load!")

    cwl_converter = BaseCWLtypes2OGCConverter(workflow)

    data = {
        "id": workflow.id.split("#")[-1],
        "version": workflow.cwlVersion,
        "inputs": cwl_converter.get_inputs(),
        "outputs": cwl_converter.get_outputs(),
    }

    if workflow.label:
        data["title"] = workflow.label

    if workflow.doc:
        data["description"] = workflow.doc

    logger.info(
        "------------------------------------------------------------------------"
    )
    logger.info("BUILD SUCCESS")
    logger.info(
        "------------------------------------------------------------------------"
    )

    logger.info(f"Saving the OCG API - Process to {output}...")

    output.parent.mkdir(parents=True, exist_ok=True)
    with output.open("w") as output_stream:
        json.dump(
            data,
            output_stream,
            indent=2,
        )

    logger.info(f"New OCG API - Process successfully saved to {output}!")

    end_time = time.time()

    logger.info(f"Total time: {end_time - start_time:.4f} seconds")
    logger.info(
        f"Finished at: {datetime.fromtimestamp(end_time).isoformat(timespec='milliseconds')}"
    )



# Copyright 2025 Terradue
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import ABC, abstractmethod
from cwl_utils.parser import (
    cwl_v1_0,
    cwl_v1_1,
    cwl_v1_2,
    CommandInputParameter,
    CommandOutputParameter,
    Directory,
    EnumSchema,
    File,
    InputArraySchema,
    InputEnumSchema,
    InputParameter,
    InputRecordSchema,
    OutputArraySchema,
    OutputEnumSchema,
    OutputParameter,
    OutputRecordSchema,
    Process,
)

from loguru import logger
from typing import (
    Any,
    get_args,
    get_origin,
    List,
    Mapping,
    MutableMapping,
    TextIO,
    Union,
)

import json

__CommandInputEnumSchema__ = Union[
    cwl_v1_0.CommandInputEnumSchema,
    cwl_v1_1.CommandInputEnumSchema,
    cwl_v1_2.CommandInputEnumSchema,
]

__CommandOutputEnumSchema__ = Union[
    cwl_v1_0.CommandOutputEnumSchema,
    cwl_v1_1.CommandOutputEnumSchema,
    cwl_v1_2.CommandOutputEnumSchema,
]

__CommandInputRecordSchema__ = Union[
    cwl_v1_0.CommandInputRecordSchema,
    cwl_v1_1.CommandInputRecordSchema,
    cwl_v1_2.CommandInputRecordSchema,
]

__CommandInputArraySchema__ = Union[
    cwl_v1_0.CommandInputArraySchema,
    cwl_v1_1.CommandInputArraySchema,
    cwl_v1_2.CommandInputArraySchema,
]

__CommandOutputArraySchema__ = Union[
    cwl_v1_0.CommandOutputArraySchema,
    cwl_v1_1.CommandOutputArraySchema,
    cwl_v1_2.CommandOutputArraySchema,
]

__CommandOutputRecordSchema__ = Union[
    cwl_v1_0.CommandOutputRecordSchema,
    cwl_v1_1.CommandOutputRecordSchema,
    cwl_v1_2.CommandOutputRecordSchema,
]

__STRING_FORMAT_URL__ = (
    "https://raw.githubusercontent.com/eoap/schemas/main/string_format.yaml"
)

__STRING_FORMATS__ = {
    "Date": "date",
    "DateTime": "date-time",
    "Duration": "duration",
    "Email": "email",
    "Hostname": "hostname",
    "IDNEmail": "idn-email",
    "IDNHostname": "idn-hostname",
    "IPv4": "ipv4",
    "IPv6": "ipv6",
    "IRI": "iri",
    "IRIReference": "iri-reference",
    "JsonPointer": "json-pointer",
    "Password": "password",
    "RelativeJsonPointer": "relative-json-pointer",
    "UUID": "uuid",
    "URI": "uri",
    "URIReference": "uri-reference",
    "URITemplate": "uri-template",
    "Time": "time",
}


class __CWLtypes2OGCConverter__(ABC):
    @abstractmethod
    def _on_enum(self, input: Any) -> Mapping[str, Any]:
        pass

    @abstractmethod
    def _on_enum_schema(self, input: Any) -> Mapping[str, Any]:
        pass

    @abstractmethod
    def _on_array(self, input: Any) -> Mapping[str, Any]:
        pass

    @abstractmethod
    def _on_input_array_schema(self, input: Any) -> Mapping[str, Any]:
        pass

    @abstractmethod
    def _on_input_parameter(self, input: Any) -> Mapping[str, Any]:
        pass

    @abstractmethod
    def _on_input(self, input: Any) -> Mapping[str, Any]:
        pass

    @abstractmethod
    def _on_list(self, input: Any) -> Mapping[str, Any]:
        pass

    @abstractmethod
    def _on_record(self, input: Any) -> Mapping[str, Any]:
        pass

    @abstractmethod
    def _on_record_schema(self, input: Any) -> Mapping[str, Any]:
        pass


class BaseCWLtypes2OGCConverter(__CWLtypes2OGCConverter__):
    """
    A helper class to automate the conversion of CWL input/output definitions into OGC API - Processes and JSON Schemas.
    """

    def __init__(self, cwl: Process):
        """
        Initializes the converter, given the CWL document where extracting informations from.

        Args:
            `cwl` (`Process`): The CWL document object model

        Returns:
            `None`: none.
        """
        self.cwl = cwl
        self._CWL_TYPES__ = {}

        def _map_type(type_: Any, map_function: Any) -> None:
            if isinstance(type_, list):
                for typ in type_:
                    _map_type(typ, map_function)
            elif get_origin(type_) is Union:
                for typ in get_args(type_):
                    _map_type(typ, map_function)
            else:
                self._CWL_TYPES__[type_] = map_function

        _map_type("int", lambda input: {"type": "integer", "format": "int32"})
        _map_type("long", lambda input: {"type": "integer", "format": "int64"})
        _map_type("double", lambda input: {"type": "number", "format": "double"})
        _map_type("float", lambda input: {"type": "number", "format": "float"})
        _map_type("boolean", lambda input: {"type": "boolean"})
        _map_type(["string", "stdout"], lambda input: {"type": "string"})

        _map_type(
            ["File", File],
            lambda input: {
                "oneOf": [
                    {"type": "string", "format": "uri"},
                    {
                        "$ref": "https://schemas.stacspec.org/v1.0.0/item-spec/json-schema/item.json"
                    },
                ]
            },
        )
        _map_type(
            ["Directory", Directory],
            lambda input: {
                "oneOf": [
                    {"type": "string", "format": "uri"},
                    {
                        "$ref": "https://schemas.stacspec.org/v1.0.0/item-spec/json-schema/item.json"
                    },
                    {
                        "$ref": "https://schemas.stacspec.org/v1.0.0/collection-spec/json-schema/collection.json"
                    },
                ]
            },
        )

        # these are not correctly interpreted as CWL types
        _map_type("record", self._on_record)
        _map_type("enum", self._on_enum)
        _map_type("array", self._on_array)

        _map_type(list, self._on_list)

        _map_type(
            [
                __CommandInputEnumSchema__,
                __CommandOutputEnumSchema__,
                EnumSchema,
                InputEnumSchema,
                OutputEnumSchema,
            ],
            self._on_enum_schema,
        )

        _map_type(
            [
                CommandInputParameter,
                CommandOutputParameter,
                InputParameter,
                OutputParameter,
            ],
            self._on_input_parameter,
        )

        _map_type(
            [
                __CommandInputArraySchema__,
                __CommandOutputArraySchema__,
                InputArraySchema,
                OutputArraySchema,
            ],
            self._on_input_array_schema,
        )

        _map_type(
            [
                __CommandInputRecordSchema__,
                __CommandOutputRecordSchema__,
                InputRecordSchema,
                OutputRecordSchema,
            ],
            self._on_record_schema,
        )

    def _clean_name(self, name: str) -> str:
        return name[name.rfind("/") + 1 :]

    def _is_nullable(self, input: Any) -> bool:
        return (
            hasattr(input, "type_")
            and isinstance(input.type_, list)
            and "null" in input.type_
        )

    # enum

    def _on_enum_internal(self, symbols: Any) -> Mapping[str, Any]:
        return {
            "type": "string",
            "enum": list(map(lambda symbol: self._clean_name(symbol), symbols)),
        }

    def _on_enum_schema(self, input: Any) -> Mapping[str, Any]:
        return self._on_enum_internal(input.type_.symbols)

    def _on_enum(self, input: Any) -> Mapping[str, Any]:
        return self._on_enum_internal(input.symbols)

    def _on_array_internal(self, items: Any) -> Mapping[str, Any]:
        return {"type": "array", "items": self._on_input(items)}

    def _on_array(self, input: Any) -> Mapping[str, Any]:
        return self._on_array_internal(input.items)

    def _on_input_array_schema(self, input: Any) -> Mapping[str, Any]:
        return self._on_array_internal(input.type_.items)

    def _on_input_parameter(self, input: Any) -> Mapping[str, Any]:
        logger.warning(f"input_parameter not supported yet: {input}")
        return {}

    def _warn_unsupported_type(self, typ: Any):
        supported_types = "\n * ".join([str(k) for k in list(self._CWL_TYPES__.keys())])
        logger.warning(
            f"{typ} not supported yet, currently supporting only:\n * {supported_types}"
        )

    def _search_type_in_dictionary(self, expected: Any) -> Mapping[str, Any]:
        for requirement in getattr(self.cwl, "requirements", []):
            if "SchemaDefRequirement" == requirement.class_:
                for type in requirement.types:
                    if expected == type.name:
                        return self._on_input(type)

        self._warn_unsupported_type(expected)
        return {}

    def _on_input(self, input: Any) -> Mapping[str, Any]:
        type: MutableMapping[str, Any] = {}

        if isinstance(input, str):
            if input in self._CWL_TYPES__:
                type.update(self._CWL_TYPES__[input](input))
            else:
                type.update(self._search_type_in_dictionary(input))
        elif hasattr(input, "type_"):
            if isinstance(input.type_, str):
                if input.type_ in self._CWL_TYPES__:
                    type.update(self._CWL_TYPES__[input.type_](input))
                else:
                    type.update(self._search_type_in_dictionary(input.type_))
            elif input.type_.__class__ in self._CWL_TYPES__:
                type.update(self._CWL_TYPES__[input.type_.__class__](input))
            else:
                self._warn_unsupported_type(input.type_)
        else:
            logger.warning(f"I still don't know what to do for {input}")

        default_value = getattr(input, "default", None)
        if default_value:
            type["default"] = default_value

        return type

    def _on_list(self, input) -> Mapping[str, Any]:
        input_list: MutableMapping[str, Any] = {"nullable": self._is_nullable(input)}

        inputs_schema = list(
            map(
                lambda item: self._on_input(item),
                filter(lambda current: "null" != current, input.type_),
            )
        )

        if 1 == len(inputs_schema):
            input_list.update(inputs_schema[0])
        else:
            input_list["anyOf"] = inputs_schema

        return input_list

    # record

    def _on_record_internal(self, record: Any, fields: List[Any]) -> Mapping[str, Any]:
        record_name = ""
        if hasattr(record, "name"):
            record_name = record.name
        elif hasattr(record, "id"):
            record_name = record.id
        else:
            logger.warning(
                f"Impossible to detect {record.__dict__}, skipping name check..."
            )

        if __STRING_FORMAT_URL__ in record_name:
            return {
                "type": "string",
                "format": __STRING_FORMATS__.get(record.name.split("#")[-1]),
            }

        record = {"type": "object", "properties": {}, "required": []}

        for field in fields:
            field_id = self._clean_name(field.name)
            record["properties"][field_id] = self._on_input(field)

            if not self._is_nullable(field):
                record["required"].append(field_id)

        return record

    def _on_record_schema(self, input: Any) -> Mapping[str, Any]:
        return self._on_record_internal(input, input.type_.fields)

    def _on_record(self, input: Any) -> Mapping[str, Any]:
        return self._on_record_internal(input, input.fields)

    def _type_to_string(self, typ: Any) -> str:
        if get_origin(typ) is Union:
            return " or ".join(
                [self._type_to_string(inner_type) for inner_type in get_args(typ)]
            )

        if isinstance(typ, list):
            return f"[ {', '.join([self._type_to_string(t) for t in typ])} ]"

        if hasattr(typ, "items"):
            return f"{self._type_to_string(typ.items)}[]"

        if hasattr(typ, "symbols"):
            return f"enum[ {', '.join([s.split('/')[-1] for s in typ.symbols])} ]"

        if hasattr(typ, "type_"):
            return self._type_to_string(typ.type_)

        if isinstance(typ, str):
            return typ

        return typ.__name__

    def _to_ogc(self, params, is_input: bool = False) -> Mapping[str, Any]:
        ogc_map = {}

        for param in params:
            schema = {
                "schema": self._on_input(param),
                "metadata": [
                    {
                        "title": "cwl:type",
                        "value": f"{self._type_to_string(param.type_)}",
                    }
                ],
            }

            if is_input:
                schema["minOccurs"] = 0 if self._is_nullable(param) else 1
                schema["maxOccurs"] = 1
                schema["valuePassing"] = "byValue"

            if param.label:
                schema["title"] = param.label

            if param.doc:
                schema["description"] = param.doc

            ogc_map[self._clean_name(param.id)] = schema

        return ogc_map

    def get_inputs(self) -> Mapping[str, Any]:
        """
        Returns a dictionary representing OGC API - Processes inputs in-memory structure.

        Returns:
            `dict`: The generated dictionary representing OGC API - Processes inputs in-memory structure.
        """
        return self._to_ogc(params=self.cwl.inputs, is_input=True)

    def get_outputs(self) -> Mapping[str, Any]:
        """
        Returns a dictionary representing OGC API - Processes outputs in-memory structure.

        Returns:
            `dict`: The generated dictionary representing OGC API - Processes inputs in-memory structure.
        """
        return self._to_ogc(params=self.cwl.outputs)

    def _to_json_schema(
        self, parameters: Mapping[str, Any], label: str
    ) -> Mapping[str, Any]:
        id = self.cwl.id.split("#")[-1]

        schema = {
            "$schema": "https://json-schema.org/draft/2020-12/schema",
            "$id": f"https://eoap.github.io/cwl2ogc/{id}/{label}.yaml",
            "description": f"The schema to represent a {id} {label} definition",
            "type": "object",
            "required": [],
            "properties": {},
            "additionalProperties": False,
            "$defs": {},
        }

        for k, v in parameters.items():
            schema["properties"][k] = {"$ref": f"#/$defs/{k}"}

            property_schema = v["schema"]
            schema["$defs"][k] = property_schema

            if "nullable" not in property_schema or not property_schema["nullable"]:
                schema["required"].append(k)

        return schema

    def get_inputs_json_schema(self) -> Mapping[str, Any]:
        """
        Returns a dictionary representing the inputs JSON Schema in-memory structure.

        Returns:
            `dict`: The generated dictionary the inputs JSON Schema in-memory structure.
        """
        return self._to_json_schema(self.get_inputs(), "inputs")

    def get_outputs_json_schema(self) -> Mapping[str, Any]:
        """
        Returns a dictionary representing the outputs JSON Schema in-memory structure.

        Returns:
            `dict`: The generated dictionary representing the outputs JSON Schema in-memory structure.
        """
        return self._to_json_schema(self.get_outputs(), "outputs")

    def _dump(self, data: Mapping[str, Any], stream: TextIO, pretty_print: bool):
        json.dump(data, stream, indent=2 if pretty_print else None)

    def dump_inputs(self, stream: TextIO, pretty_print: bool = False):
        """
        Dumps the OGC API - Processes inputs schema to its JSON representation.

        Args:
            `stream` (`TextIO`): The stream where serializing the JSON representation
            `pretty_print` (`bool`): formats the output if `True`, in a single line otherwise. Default is `False`

        Returns:
            `None`: none.
        """
        self._dump(data=self.get_inputs(), stream=stream, pretty_print=pretty_print)

    def dump_outputs(self, stream: TextIO, pretty_print: bool = False):
        """
        Dumps the OGC API - Processes outputs schema to its JSON representation.

        Args:
            `stream` (`TextIO`): The stream where serializing the JSON representation
            `pretty_print` (`bool`): formats the output if `True`, in a single line otherwise. Default is `False`

        Returns:
            `None`: none.
        """
        self._dump(data=self.get_outputs(), stream=stream, pretty_print=pretty_print)

    def dump_inputs_json_schema(self, stream: TextIO, pretty_print: bool = False):
        """
        Dumps the inputs JSON Schema to its JSON representation.

        Args:
            `stream` (`TextIO`): The stream where serializing the JSON representation
            `pretty_print` (`bool`): formats the output if `True`, in a single line otherwise. Default is `False`

        Returns:
            `None`: none.
        """
        self._dump(
            data=self.get_inputs_json_schema(), stream=stream, pretty_print=pretty_print
        )

    def dump_outputs_json_schema(self, stream: TextIO, pretty_print: bool = False):
        """
        Dumps the outputs JSON Schema to its JSON representation.

        Args:
            `stream` (`TextIO`): The stream where serializing the JSON representation
            `pretty_print` (`bool`): formats the output if `True`, in a single line otherwise. Default is `False`

        Returns:
            `None`: none.
        """
        self._dump(
            data=self.get_outputs_json_schema(),
            stream=stream,
            pretty_print=pretty_print,
        )



# Copyright 2025 Terradue
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import io
import json
from pathlib import Path

import pytest
import yaml
from click.testing import CliRunner
from cwl_utils.parser import load_document_by_yaml

from cwl2ogc import BaseCWLtypes2OGCConverter
from cwl2ogc.cli import main


ARTIFACTS_DIR = Path("tests/artifacts")
CWL_TYPES_DIR = ARTIFACTS_DIR / "cwl-types"
CWL_TYPES_FIXTURES = [
    "inp",
    "array-inputs",
    "record",
    "exclusive-parameter-expressions",
    "complex-cwl-types",
]


def load_cwl_document(path: Path):
    with path.open() as stream:
        cwl_content = yaml.load(stream, Loader=yaml.SafeLoader)
    return load_document_by_yaml(yaml=cwl_content, uri="io://", load_all=True)


def load_json(path: Path):
    with path.open() as stream:
        return json.load(stream)


@pytest.mark.parametrize("fixture_name", CWL_TYPES_FIXTURES)
def test_conversion_matches_golden_files(fixture_name: str):
    workflow = load_cwl_document(CWL_TYPES_DIR / f"{fixture_name}.cwl")
    converter = BaseCWLtypes2OGCConverter(workflow)

    expected_inputs = load_json(CWL_TYPES_DIR / f"{fixture_name}_inputs.json")
    expected_outputs = load_json(CWL_TYPES_DIR / f"{fixture_name}_outputs.json")

    actual_inputs = converter._to_ogc(workflow.inputs)
    actual_outputs = converter._to_ogc(workflow.outputs)

    assert set(actual_inputs) == set(expected_inputs)
    assert set(actual_outputs) == set(expected_outputs)

    for name, expected_entry in expected_inputs.items():
        actual_entry = actual_inputs[name]
        if "title" in expected_entry:
            assert actual_entry.get("title") == expected_entry["title"]
        if "description" in expected_entry:
            assert actual_entry.get("description") == expected_entry["description"]

        expected_schema = expected_entry.get("schema", {})
        actual_schema = actual_entry.get("schema", {})
        assert isinstance(actual_schema, dict)
        if expected_schema:
            assert actual_schema

    for value in actual_inputs.values():
        assert "metadata" in value


def test_workflow_graph_conversion_for_water_bodies():
    cwl_graph = load_cwl_document(ARTIFACTS_DIR / "app-water-body.1.1.0.cwl")
    assert isinstance(cwl_graph, list)

    workflow = next(
        entry for entry in cwl_graph if getattr(entry, "class_", None) == "Workflow"
    )
    converter = BaseCWLtypes2OGCConverter(workflow)

    inputs = converter.get_inputs()
    outputs = converter.get_outputs()

    assert "aoi" in inputs
    assert inputs["aoi"]["schema"]["type"] == "string"
    assert inputs["aoi"]["minOccurs"] == 1
    assert outputs["stac_catalog"]["schema"]["oneOf"]


def test_json_schema_generation_for_nullable_inputs():
    workflow = load_cwl_document(CWL_TYPES_DIR / "inp.cwl")
    converter = BaseCWLtypes2OGCConverter(workflow)

    inputs_schema = converter.get_inputs_json_schema()
    outputs_schema = converter.get_outputs_json_schema()

    assert inputs_schema["$schema"] == "https://json-schema.org/draft/2020-12/schema"
    assert "example_file" in inputs_schema["$defs"]
    assert "example_file" not in inputs_schema["required"]
    assert "example_int" in inputs_schema["required"]
    assert outputs_schema["type"] == "object"


def test_dump_methods_emit_valid_json():
    workflow = load_cwl_document(CWL_TYPES_DIR / "record.cwl")
    converter = BaseCWLtypes2OGCConverter(workflow)

    streams = [io.StringIO(), io.StringIO(), io.StringIO(), io.StringIO()]
    converter.dump_inputs(streams[0], pretty_print=True)
    converter.dump_outputs(streams[1], pretty_print=True)
    converter.dump_inputs_json_schema(streams[2], pretty_print=True)
    converter.dump_outputs_json_schema(streams[3], pretty_print=True)

    for stream in streams:
        data = json.loads(stream.getvalue())
        assert isinstance(data, dict)


def test_cli_writes_process_json(tmp_path: Path):
    output = tmp_path / "process.json"
    source = CWL_TYPES_DIR / "inp.cwl"

    runner = CliRunner()
    result = runner.invoke(
        main,
        [str(source), "--workflow-id", "inp", "--output", str(output)],
    )

    assert result.exit_code == 0
    assert output.exists()

    process = load_json(output)
    assert process["id"] == "inp"
    assert "inputs" in process
    assert "outputs" in process



import os
import json
import streamlit as st
from cwl2ogc import load_converter_from_string_content
from code_editor import code_editor
import tempfile
from io import StringIO
import yaml
from cwltool.main import main as cwltool

st.header("CWL to OGC API Processes inputs/outputs")
st.set_page_config(layout="wide")


def validate_cwl(cwl_content):
    """Checks whether the CWL file meets basic conformance criteria.

    Returns
    -------
    tuple
        A tuple containing the return value of cwltool and
        the stdout and stderr content
    """
    temp_dir = tempfile.mkdtemp()
    temp_cwl_path = os.path.join(temp_dir, "temp_cwl")

    with open(temp_cwl_path, "w") as outfile:
        yaml.dump(cwl_content, outfile, default_flow_style=False)

    out = StringIO()
    err = StringIO()
    res = cwltool(
        ["--validate", temp_cwl_path],
        stderr=out,
        stdout=err,
    )
    os.remove(temp_cwl_path)

    return res, out.getvalue(), err.getvalue()


btn_settings_editor_btns = [
    {
        "name": "copy",
        "feather": "Copy",
        "hasText": True,
        "alwaysOn": True,
        "commands": ["copyAll"],
        "style": {"top": "0rem", "right": "0.4rem"},
    },
    {
        "name": "update",
        "feather": "RefreshCw",
        "primary": True,
        "hasText": True,
        "showWithIcon": True,
        "commands": ["submit"],
        "style": {"bottom": "0rem", "right": "0.4rem"},
    },
]

with open(
    os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        "resources/custom_buttons_bar_alt.json",
    )
) as json_button_file_alt:
    custom_buttons_alt = json.load(json_button_file_alt)

height = [22, 25]
language = "yaml"
theme = "default"
shortcuts = "vscode"
focus = False
wrap = True
btns = custom_buttons_alt

example_cwl = """\
cwlVersion: v1.2
class: CommandLineTool
id: array-inputs
inputs:
  filesA:
    label: filesA label
    doc: filesA doc
    type: string[]
    inputBinding:
      prefix: -A
      position: 1

  filesB:
    label: filesB label
    doc: filesB doc
    type:
      type: array
      items: string
      inputBinding:
        prefix: -B=
        separate: false
    inputBinding:
      position: 2

  filesC:
    label: filesC label
    doc: filesC doc
    type: string[]
    inputBinding:
      prefix: -C=
      itemSeparator: ","
      separate: false
      position: 4

outputs:
  example_out:
    type: stdout
stdout: output.txt
baseCommand: echo
"""

ace_props = {"style": {"borderRadius": "0px 0px 8px 8px"}}
response_dict = code_editor(
    example_cwl,
    height=height,
    lang=language,
    theme=theme,
    shortcuts=shortcuts,
    focus=focus,
    buttons=btns,
    props=ace_props,
    options={"wrap": wrap},
    allow_reset=True,
    key="code_editor_demo",
)

if response_dict["type"] == "submit":
    cwl_content = response_dict["text"]

    try:
        res, out, err = validate_cwl(yaml.load(cwl_content, Loader=yaml.FullLoader))

        if res != 0:
            st.error(f"Validation failed with error code {res}.")
            st.text_area("Validation Output", value=out, height=10)
            st.text_area("Validation Error", value=err, height=300)
            st.stop()

        converter = load_converter_from_string_content(cwl_content)

        inputs = converter.get_inputs()
        outputs = converter.get_outputs()
        inputs_schema = converter.get_inputs_json_schema()
        outputs_schema = converter.get_outputs_json_schema()

        col1, col2, col3, col4 = st.columns(4)
        with col1:
            st.subheader("Inputs OGC definition")
            st.json(inputs)

        with col2:
            st.subheader("Outputs OGC definition")
            st.json(outputs)

        with col3:
            st.subheader("Inputs JSON schema")
            st.json(inputs_schema)

        with col4:
            st.subheader("Outputs JSON schema")
            st.json(outputs_schema)

    except Exception as e:
        st.error(f"Error parsing CWL: {e}")
